"""Higher level module to handle data processing of BEA API results"""

import pandas as pd

from . import api


class BEA(api.BEAAPI):
    r"""The primary class in the BEApy package, handles sending requests
    and manipulating the results. Needs to be initiated with your BEA
    API key once, it is stored for all later requests.

    Parameters
    ----------
    api_key
        Your BEA API key
    """

    def __init__(self, api_key: str) -> None:
        super().__init__(api_key)

    def dataset_list(self) -> pd.DataFrame:
        r"""Get a list of all availible datasets (API method ``getDatasetList``)

        Returns
        -------
        pd.DataFrame
            A DataFrame with a description and name of availible datasets.
        """

        params = {
            'method': 'GETDATASETLIST'
        }

        results = self.send_request(params)

        return pd.DataFrame(results['Dataset'])

    def parameter_list(self, dataset: str) -> pd.DataFrame:
        r"""Get a list of availible parameters for the dataset  (API method ``getParameterList``)

        Usually the parameter list will also include the value of the ALL option,
        as well as data type, whether the parameteris is required, and
        whether it accepts multiple values.

        Parameters
        ----------
        dataset
            The name of the dataset

        Returns
        -------
        pd.DataFrame
            A DataFrame with a description and name of availible parameters,
            as well as some other options.
        """

        params = {
            'method': 'GETPARAMETERLIST',
            'datasetname': dataset,
        }

        results = self.send_request(params)['Parameter']

        # If we get a dict, there was only one entry, we ned to provide set_index manually
        if type(results) == dict:
            parameters = pd.DataFrame(results, index = [0])
        else:
            parameters = pd.DataFrame(results)

        # Shorten names
        parameters.columns = [x.replace('Parameter', '') for x in parameters.columns]
        parameters.columns = [x.replace('Flag', '') for x in parameters.columns]
        parameters['Name'] = parameters['Name'].str.strip()

        # Set name as index
        parameters.set_index('Name', inplace = True)

        # Default column order - filter columns not availible, append others
        default_order = ['Description', 'DefaultValue', 'DataType',
                         'IsRequired', 'MultipleAccepted', 'AllValue']
        order = [x for x in default_order if x in parameters.columns]
        extra = [x for x in parameters.columns if x not in order]

        order = order + extra

        # Reorder
        parameters = parameters[order]

        # Columns to bool
        parameters['MultipleAccepted'] = parameters['MultipleAccepted']\
                                                .astype(int).astype(bool)
        parameters['IsRequired'] = parameters['IsRequired']\
                                                .astype(int).astype(bool)

        return parameters
