import os

import pytest
import pandas as pd

from beapy import BEA
from beapy.api import BEAError


def test_wrong_id():
    """Test wrong ID, should catch a BEAError"""
    bea = BEA('WRONG-KEY')
    with pytest.raises(BEAError):
        bea.send_request({})

# From now on, the correct ID is used, so test can share a BEA object
@pytest.fixture(scope='session')
def api_key():
    return os.environ['BEA_API_KEY']


@pytest.fixture(scope='module')
def bea(api_key):
    bea = BEA(api_key)

    return bea


def test_wrong_param(bea):
    """Test wrong params, should catch a BEAError"""
    with pytest.raises(BEAError):
        bea.send_request({'WRONG PARAM': 0})


def test_wrong_method(bea):
    """Test wrong method name, should catch a BEAError"""
    with pytest.raises(BEAError):
        bea.send_request({'method': 'WRONG METHOD'})


@pytest.fixture(scope='module')
def dataset_list(bea):
    return bea.dataset_list()


class TestDatasetList(object):
    """This makes sure that the output of dataset_list() is what you
    would expect: a non-empty pd.DataFrame with 2 columns, one named
    'DatasetDescription', the other 'DatasetName'"""

    def test_type(self, dataset_list):
        assert isinstance(dataset_list, pd.DataFrame)

    def test_num_cols(self, dataset_list):
        assert dataset_list.shape[1] == 2

    def test_nonempty(self, dataset_list):
        assert dataset_list.shape[0] > 0

    def test_descr_column(self, dataset_list):
        assert 'DatasetDescription' in dataset_list.columns

    def test_name_column(self, dataset_list):
        assert 'DatasetName' in dataset_list.columns


class TestParameterList(object):
    """This makes sure that the output of parameter_list() if what we would
    expect for all the current databases (non-empty dataframe), and tests the
    presence of some parameters for the NIPA and MNE databases"""

    def test_wrong_database(self, bea):
        with pytest.raises(BEAError):
            bea.parameter_list('WRONG DATABASE')

    def test_default_datasets_general(self, bea, dataset_list):
        """This tests the output for all datasets, to make sure it is
        a DataFrame, not empty, and contains the Description columns"""

        db_list = list(dataset_list['DatasetName'])

        for dataset in db_list:
            params = bea.parameter_list(dataset)


            error_not_df = f'{dataset} parameter list is not a DataFrame'
            error_empty = f'{dataset} parameter list is empty'
            error_no_desc = f'{dataset} parameter list does not contain a Description column'

            assert isinstance(params, pd.DataFrame), error_not_df
            assert params.shape[0] > 0, error_empty
            assert 'Description' in params.columns, error_no_desc

    def test_NIPA(self, bea):
        """This makes sure that NIPA contains the parameters and columns it should"""

        parameters = ['Frequency', 'ShowMillions', 'TableID', 'TableName', 'Year']
        columns = ['Description', 'DefaultValue', 'DataType', 'IsRequired', 'MultipleAccepted',
                   'AllValue']

        NIPA = bea.parameter_list('NIPA')

        assert set(parameters) == set(NIPA.index), 'Parameter list for NIPA not ok'
        assert set(columns) == set(NIPA.columns), 'Parameter attribute list (cols) for NIPA not ok'

    def test_MNE(self, bea):
        """This makes sure that MNE contains the parameters and columns it should"""

        parameters = ['DirectionOfInvestment', 'OwnershipLevel', 'NonbankAffiliatesOnly',
                      'Classification', 'Country', 'Industry', 'Year', 'State', 'SeriesID',
                      'GetFootnotes', 'Investment', 'ParentInvestment']
        columns = ['Description', 'DefaultValue', 'DataType', 'IsRequired', 'MultipleAccepted',
                   'AllValue']

        MNE = bea.parameter_list('MNE')

        assert set(parameters) == set(MNE.index), 'Parameter list for MNE not ok'
        assert set(columns) == set(MNE.columns), 'Parameter attribute list (cols) for MNE not ok'
